"""
Find the locus, gene, upstream, etc. data
using the csv output from IntaRNA.
Output as a .csv so it can be copy pasted
into excel.
"""

import os

def getInteraction(bpList):
    interaction = ""
    bpList = bpList.split(":")
    start = int(bpList[0].split(",")[0][1:])
    end = int(bpList[-1].split(",")[0][1:])
    num_bases = end - start + 1
    base_pairs = [0] * num_bases
    for i in range(len(bpList)):
        index = int(bpList[i].split(",")[0][1:]) - start
        base_pairs[index] = 1
    j = 0
    for num in base_pairs:
        if j == len(base_pairs) // 2:
            interaction += "&"
        elif num == 1 and j < len(base_pairs) / 2:
            interaction += "("
        elif num == 1 and j > len(base_pairs) / 2:
            interaction += ")"
        else:
            interaction += "."
        j += 1
    return interaction

def outfileWriteHeader(outfile):
    outfile.write("sep=;\n")
    description = "sRNA_ID;sRNA_subseq_start;sRNA_subseq_end;target;target_start;target_end;mRNA_subseq_start;mRNA_subseq_end;energy;probability;annotation;upstream_gene;upstream_start;upstream_end;downstream_gene;downstream_start;downstream_end;interaction\n"
    outfile.write(description)
    return

def outfileWriteData(data, outfile):
    id2, target, gene_start, gene_end, start1, end1, start2, end2, energy, prob, annotation, up, down, interaction = data
    upstream_target, up_start, up_end = up
    downstream_target, down_start, down_end = down
    outfile.write(id2 + ";")
    outfile.write(start2 + ";")
    outfile.write(end2 + ";")
    outfile.write(target + ";")
    outfile.write(gene_start + ";")
    outfile.write(gene_end.strip() + ";")
    outfile.write(start1 + ";")
    outfile.write(end1 + ";")
    outfile.write(energy + ";")
    outfile.write(prob + ";")
    outfile.write(annotation + ";")
    outfile.write(upstream_target + ";")
    outfile.write(up_start + ";")
    outfile.write(up_end.strip() + ";")
    outfile.write(downstream_target + ";")
    outfile.write(down_start + ";")
    outfile.write(down_end.strip() + ";")
    outfile.write(interaction + "\n")
    return

def getUpstream(infile):
    upstream = ""
    line = infile.readline()
    while line[5:9] != "gene":
        line = infile.readline()
    gene_start, gene_end = line[21:].split("..")
    if "comp" in gene_start:
        gene_start = gene_start.split("(")[1]
        gene_end = gene_end.split(")")[0]
    if ">" in gene_end:
        gene_end = gene_end[1:]
    if "<" in gene_start:
        gene_start = gene_start[1:]
    line= infile.readline()
    while "/product=" not in line:
        line = infile.readline()
    i = 31
    while line[i] != '"':
        if line[i] == "\n":
            line = infile.readline()
            i = 21
            upstream += " "
        upstream += line[i]
        i += 1
    return upstream, gene_start, gene_end
    
def getDownstream(infile):
    downstream = ""
    line = infile.readline()
    while line[5:9] != "gene":
        line = infile.readline()
    gene_start, gene_end = line[21:].split("..")
    if "comp" in gene_start:
        gene_start = gene_start.split("(")[1]
        gene_end = gene_end.split(")")[0]
    if ">" in gene_end:
        gene_end = gene_end[1:]
    if "<" in gene_start:
        gene_start = gene_start[1:]
    line = infile.readline()
    while "/product=" not in line:
        line = infile.readline()
    i = 31
    while line[i] != '"':
        if line[i] == "\n":
            line = infile.readline()
            i = 21
            downstream += " "
        downstream += line[i]
        i += 1
    return downstream, gene_start, gene_end

def getTarget(infile, mRNA_start, mRNA_end):
    annotation = ""
    infile.seek(0)
    prev = 0
    while "FEATURES" not in infile.readline():
        continue
    while 1:
        line = infile.readline()
        if line[5:9] == "gene":
            curr = infile.tell()
            gene_start, gene_end = line[21:].split("..")
            if "comp" in gene_start:
                gene_start = gene_start.split("(")[1]
                gene_end = gene_end.split(")")[0]
            if ">" in gene_end:
                gene_end = gene_end[1:]
            if "<" in gene_start:
                gene_start = gene_start[1:]
            if (mRNA_end >= int(gene_start) and mRNA_end <= int(gene_end)) or (mRNA_start >= int(gene_start) and mRNA_start <= int(gene_end)):
                line = infile.readline()
                while line[21] == "/" and line[22] != "p":
                    if "gene" in line:
                        target = line[21:].split('"')[1]
                        break
                    elif "old" in line:
                        target = line[21:].split('"')[1]
                        break
                    else:
                        target = line[21:].split('"')[1]
                    line = infile.readline()
                line = infile.readline()
                while "/product=" not in line:
                    line = infile.readline()
                i = 31
                while line[i] != '"':
                    if line[i] == "\n":
                        line = infile.readline()
                        i = 21
                        annotation += " "
                    annotation += line[i]
                    i += 1
                infile.seek(prev - 100)
                up = getUpstream(infile)
                infile.seek(curr)
                down = getDownstream(infile)
                return target, annotation, gene_start, gene_end, up, down
            if mRNA_start < int(gene_start):
                target = "non-coding"
                infile.seek(prev - 100)
                up = getUpstream(infile)
                down = getDownstream(infile)
                return target, "N/A", "N/A", "N/A", up, down
            prev = curr
        if not line:
            break
    return "", "", "", ""

def main():
    path = os.getcwd()[:-7]
    intaRNA = path + "IntaRNA stuff/"
    strains = ["D39",
               "TIGR4",
               "TVO_1901920",
               "TVO_1901934",
               "TVO_1902277",
               "TVO_Taiwan19F-14"]
    for strain in strains:
        csv = intaRNA + strain + " files/" + strain + "_results.csv"
        genbank = path + "Annotated genomes/" + strain + ".gb"
        top_hit = intaRNA + strain + " files/IntaRNATop_Hit.csv"
        top_5_hits = intaRNA + strain + " files/IntaRNATop_5_Hits.csv"
        infile1 = open(csv)
        infile2 = open(genbank)
        outfile1 = open(top_hit, "w")
        outfile2 = open(top_5_hits, "w")
    
        infile1.readline()
        i = 0
        line = infile1.readline().strip()
        outfileWriteHeader(outfile1)
        outfileWriteHeader(outfile2)
        while line:
            id1, id2, start1, end1, start2, end2, subseq1, subseq2, bpList, energy, prob = line.split(";")
            target, annotation, gene_start, gene_end, up, down = getTarget(infile2, int(start1), int(end1))
            interaction = getInteraction(bpList)
            data = (id2[:-1], target, gene_start, gene_end, start1, end1, start2, end2, energy, prob,
                    annotation, up, down, interaction)
            outfileWriteData(data, outfile2)
            if i % 5 == 0:
                outfileWriteData(data, outfile1)
            i += 1
            line = infile1.readline().strip() 
    
        infile1.close()
        infile2.close()
        outfile1.close()
        outfile2.close()

main()