"""
Find the upstream, downstream, and target length
of the predicted target
Output as a .csv so it can be copy pasted
into excel.
"""

import os

def outfileWriteHeader(outfile):
    outfile.write("sep=;\n")
    description = "sRNA_ID;sRNA_subseq_start;sRNA_subseq_end;target;target_length;target_start;target_end;mRNA_subseq_start;mRNA_subseq_end;energy;P-value;probability;annotation;upstream_gene;upstream_start;upstream_end;downstream_gene;downstream_start;downstream_end;interaction\n"
    outfile.write(description)
    return

def outfileWriteData(data, outfile):
    sRNA_id, sRNA_start, sRNA_stop, target, target_length, gene_start, gene_end, target_start, target_end, energy, p_value, probability, annotation, up, down, interaction = data
    upstream_target, up_start, up_end = up
    downstream_target, down_start, down_end = down
    outfile.write(sRNA_id + ";")
    outfile.write(sRNA_start + ";")
    outfile.write(sRNA_stop + ";")
    outfile.write(target + ";")
    outfile.write(target_length + ";")
    outfile.write(gene_start + ";")
    outfile.write(gene_end.strip() + ";")
    outfile.write(target_start + ";")
    outfile.write(target_end + ";")
    outfile.write(energy + ";")
    outfile.write(p_value + ";")
    outfile.write(probability + ";")
    outfile.write(annotation + ";")
    outfile.write(upstream_target + ";")
    outfile.write(up_start + ";")
    outfile.write(up_end.strip() + ";")
    outfile.write(downstream_target + ";")
    outfile.write(down_start + ";")
    outfile.write(down_end.strip() + ";")
    outfile.write(interaction + "\n")
    return

def getUpstream(infile):
    upstream = ""
    line = infile.readline()
    while line[5:9] != "gene":
        line = infile.readline()
    gene_start, gene_end = line[21:].split("..")
    if "comp" in gene_start:
        gene_start = gene_start.split("(")[1]
        gene_end = gene_end.split(")")[0]
    if ">" in gene_end:
        gene_end = gene_end[1:]
    if "<" in gene_start:
        gene_start = gene_start[1:]
    line= infile.readline()
    while "/product=" not in line:
        line = infile.readline()
    i = 31
    while line[i] != '"':
        if line[i] == "\n":
            line = infile.readline()
            i = 21
            upstream += " "
        upstream += line[i]
        i += 1
    return upstream, gene_start, gene_end
    
def getDownstream(infile):
    downstream = ""
    line = infile.readline()
    while line[5:9] != "gene":
        line = infile.readline()
    gene_start, gene_end = line[21:].split("..")
    if "comp" in gene_start:
        gene_start = gene_start.split("(")[1]
        gene_end = gene_end.split(")")[0]
    if ">" in gene_end:
        gene_end = gene_end[1:]
    if "<" in gene_start:
        gene_start = gene_start[1:]
    line = infile.readline()
    while "/product=" not in line:
        line = infile.readline()
    i = 31
    while line[i] != '"':
        if line[i] == "\n":
            line = infile.readline()
            i = 21
            downstream += " "
        downstream += line[i]
        i += 1
    return downstream, gene_start, gene_end

def getTarget(infile, target):
    if "(" in target:
        temp = target.split("(")[-1]
        target = temp[:-1]
    infile.seek(0)
    prev = 0
    while "FEATURES" not in infile.readline(): continue
    while 1:
        line = infile.readline()
        if line[5:9] == "gene":
            curr = infile.tell()
            gene_start, gene_end = line[21:].split("..")
            if "comp" in gene_start:
                gene_start = gene_start.split("(")[1]
                gene_end = gene_end.split(")")[0]
            if ">" in gene_end:
                gene_end = gene_end[1:]
            if "<" in gene_start:
                gene_start = gene_start[1:]
            line = infile.readline()
            i = 21
            ids = []
            while line[i] == "/":
                ids.append(line)
                line = infile.readline()
            found = False
            for id in ids:
                if target in id:
                    found = True
            if found:
                line = infile.readline()
                while "/product=" not in line:
                    line = infile.readline()
                i = 31
                while line[i] != '"':
                    if line[i] == "\n":
                        line = infile.readline()
                        i = 21
                    i += 1
                if prev - 100 >= 0:
                    infile.seek(prev - 100)
                    up = getUpstream(infile)
                else:
                    up = "N/A", "N/A", "N/A"
                infile.seek(curr)
                down = getDownstream(infile)
                length = int(gene_end) - int(gene_start) + 1
                return str(length), gene_start, gene_end, up, down
            prev = curr
        if not line:
            break
    return

def main():
    path = os.getcwd()[:-7]
    targetrna = path + "TargetRNA3 stuff/"
    strains = ["D39", "TIGR4"]
    for strain in strains:
        tsv = targetrna + strain + "_results/" + strain + "_summary.tsv"
        genbank = path + "Annotated genomes/" + strain + ".gb"
        data = targetrna + strain + "_results/" + strain + "_updated.csv"
        infile1 = open(tsv)
        infile2 = open(genbank)
        outfile = open(data, "w")
        
        line = infile1.readline().strip()
        outfileWriteHeader(outfile)
        
        while line:
            sRNA_id, index, target, energy, p_value, probability, sRNA_start, sRNA_stop, target_start, target_end, annotation, interaction = line.split("\t")
            target_length, gene_start, gene_end, up, down = getTarget(infile2, target)
            data = (sRNA_id, sRNA_start, sRNA_stop, target, target_length, gene_start, gene_end,
                    target_start, target_end, energy, p_value, probability, annotation, 
                    up, down, interaction)
            outfileWriteData(data, outfile)
            line = infile1.readline().strip() 
    
        infile1.close()
        infile2.close()
        outfile.close()

main()