"""
Evaluate the synteny of the sRNAs by comparing
the sequences 1000 nt up and downstream of the
sRNA sequences for each of the 385 strains
"""

import os
import helper_functions as hf

def compareSeqs(seq1, seq2):
    mismatch = 0
    for i in range(min(len(seq1), len(seq2))):
        if seq1[i] != seq2[i]:
            mismatch += 1
    return round((1 - mismatch / len(seq1)) * 100, 2)

def getCoords(file, sRNA):
    with open(file) as infile:
        # Confirm there is a hit for the sRNA first
        hit = False
        line = infile.readline()
        while line[0] != ">":
            if sRNA + ":" in line:
                hit = True
            line = infile.readline()
        infile.seek(infile.tell() - 100)
        if not hit:
            return None, None
        line = infile.readline()
        while line.split(":")[0] != (">" + sRNA):
            line = infile.readline()
        for i in range(6):
            infile.readline()
        _, start, _, temp = infile.readline().split("  ")
        while True:
            for i in range(3):
                infile.readline()
            query = infile.readline()
            if "Query" not in query:
                break
            last_query = query
        try:
            end = last_query.strip().split("  ")[-1] or temp
        except:
            end = temp
    return start, end

def main():
    path = os.getcwd()[:-7]
    fastas = []
    for filename in os.listdir(path + "Other fasta files"):
        if filename[-6:] == ".fasta" and "ignore" not in filename:
            fastas.append(filename)
    for filename in os.listdir(path + "TVO fasta files"):
        if filename[-6:] == ".fasta":
            fastas.append(filename)
    for filename in os.listdir(path + "Clinical fasta files"):
        if filename[-6:] == ".fasta":
            fastas.append(filename)

    sRNAs = []
    with open(path + "sRNA data/In-house_sRNAsV3.fasta") as infile:
        line = infile.readline()
        while line:
            sRNAs.append(line[1:].split(":")[0])
            for i in range(2):
                infile.readline()
            line = infile.readline()

    outfile = open(path + "sRNA data/synteny_results.txt", "w")
    outfile.write("sRNA, Is the sequence preserved (Yes/No)")
    for sRNA in sRNAs:
        avg_up = 0
        avg_down = 0
        strains = 0
        with open(path + "Other fasta files/TIGR4.fasta") as infile:
            start, end = getCoords(path + "sRNA BLAST search results/TIGR4results.txt", sRNA)
            TIGR4_up = hf.retrieve_sequence(int(start) - 1000, int(start) - 1, infile)
            TIGR4_down = hf.retrieve_sequence(int(end) + 1, int(end) + 1000, infile)
        for strain in fastas:
            file = path + "sRNA BLAST search results/" + strain[:-6] + "results.txt"
            start, end = getCoords(file, sRNA)
            if start == None and end == None:
                continue
            strains += 1
            if "#" in strain:
                with open(path + "Clinical fasta files/" + strain) as infile:
                    up = hf.retrieve_sequence(int(start) - 1000, int(start) - 1, infile)
                    down = hf.retrieve_sequence(int(end) + 1, int(end) + 1000, infile)
            elif "TVO" in strain:
                with open(path + "TVO fasta files/" + strain) as infile:
                    up = hf.retrieve_sequence(int(start) - 1000, int(start) - 1, infile)
                    down = hf.retrieve_sequence(int(end) + 1, int(end) + 1000, infile)
            else:
                with open(path + "Other fasta files/" + strain) as infile:
                    up = hf.retrieve_sequence(int(start) - 1000, int(start) - 1, infile)
                    down = hf.retrieve_sequence(int(end) + 1, int(end) + 1000, infile)
            avg_up += compareSeqs(TIGR4_up, up)
            avg_down += compareSeqs(TIGR4_down, down)
        avg_up /= strains
        avg_down /= strains
        outfile.write(sRNA)
        if avg_up >= 75:
            outfile.write(", Yes")
        else:
            outfile.write(", No")
        if avg_down >= 75:
            outfile.write(", Yes\n")
        else:
            outfile.write(", No\n")
    outfile.close()

main()