"""
Get the sequences of the sRNA-mRNA interaction
including 25 nucleotides upstream and downstream
of the sequence.
These sequences will be used by RNAFold to
determine if the base pairing is occurring in
regions in which the mRNA is folding.
"""

import os
import helper_functions as hf

def outfileWrite(outfile, data):
    name, sequence = data
    outfile.write(">" + name + "\n")
    outfile.write(sequence + "\n")
    outfile.write("\n")
    return

def main():
    path = os.getcwd()[:-7]
    intaRNA = path + "IntaRNA stuff/"
    strains = ["D39",
               "TIGR4",
               "TVO_1901920",
               "TVO_1901934",
               "TVO_1902277",
               "TVO_Taiwan19F-14"]
    for strain in strains:
        csv = intaRNA + strain + " files/" + strain + "_results.csv"
        if strain == "D39" or strain == "TIGR4":
            directory = path + "Other fasta files/"
        else:
            directory = path + "TVO fasta files/"
        fasta = directory + strain + ".fasta"
        output = intaRNA + strain + " files/" + "mRNA_target_+25.fasta"
        infile1 = open(csv)
        infile2 = open(fasta)
        outfile = open(output, "w")
        infile1.readline()
        line = infile1.readline().strip()
        while line:
            id1, id2, start1, end1, start2, end2, subseq1, subseq2, bpList, energy, prob = line.split(";")
            sequence = hf.retrieve_sequence(int(start1), int(end1), infile2, 25, 25)
            outfileWrite(outfile, (id2[:-1], sequence))
            for _ in range(5):
                line = infile1.readline().strip() 
    
        infile1.close()
        infile2.close()
        outfile.close()

main()