"""
Count the average number of hits for each sRNA.
A high number of average hits would indicate the
sRNA is potentially a BOX element/transposon.
Also report the average identity %.
Output the numbers to a file called sRNA_similarity.txt.
"""

import os

def main():
    path = os.getcwd()[:-7]
    blast_results = path + "sRNA BLAST search results"
    outfilename = path + "sRNA data/sRNA_similarity.tsv"
    
    sRNAs = dict()
    
    for filename in os.listdir(blast_results):
        if filename[-11:] == "results.txt":
            file = blast_results + "/" + filename
            with open(file) as infile:
                while "Sequences" not in infile.readline(): continue
                infile.readline()
                line = infile.readline()
                while line[0] == "M":
                    id = line.split(":")[0]
                    if id not in sRNAs:
                        sRNAs[id] = [0, 0.0, 1, 0, 0]
                        # index 0 = # hits
                        # index 1 = identity %
                        # index 2 = # genomes present in
                        # index 3 = # same strand hits
                        # index 4 = best alignment identity
                    else:
                        sRNAs[id][2] += 1
                    line = infile.readline()
                infile.readline()
                line = infile.readline()
                while line[0] == ">":
                    save = True
                    id, ignore, sRNA_strand = line.split(" ")
                    id = id[1:-1]
                    sRNA_strand = sRNA_strand.strip()
                    for _ in range(2): infile.readline()
                    line = infile.readline()
                    while line[0] == " ":
                        line = infile.readline()
                        if "Identities" in line:
                            sRNAs[id][0] += 1
                        identity = line[1:].split(" ")[2]
                        numerator, denominator = identity.split("/")
                        sRNAs[id][1] += (int(numerator) / int(denominator)) * 100
                        if save:
                            sRNAs[id][4] += (int(numerator) / int(denominator)) * 100
                            save = False
                        genome_strand = infile.readline().strip().split("/")[1]
                        if (genome_strand == "Minus" and sRNA_strand == "-") or (genome_strand == "Plus" and sRNA_strand == "+"):
                            sRNAs[id][3] += 1
                        infile.readline()
                        while "Query" in infile.readline():
                            for _ in range(3): infile.readline()
                        line = infile.readline()
    # for id in sRNAs:
    #     sRNAs[id][4] /= sRNAs[id][2]
    #     print(f"{id} {round(sRNAs[id][4], 2)}")
    with open(outfilename, "w") as outfile:
        outfile.write("ID\tAverage_#_hits\tAverage_identity_%\t#_genomes_present_in\t#_same_strand\tAverage_best_alignment_identity\n")
        for id in sRNAs:
            sRNAs[id][1] /= sRNAs[id][0]
            sRNAs[id][0] /= sRNAs[id][2]
            sRNAs[id][3] /= sRNAs[id][0]
            sRNAs[id][4] /= sRNAs[id][2]
            outfile.write(id + "\t")
            outfile.write(str(round(sRNAs[id][0], 1)) + "\t")
            outfile.write(str(round(sRNAs[id][1], 2)) + "\t")
            outfile.write(str(sRNAs[id][2]) + "\t")
            outfile.write(str(int(sRNAs[id][3])))
            outfile.write(str(int(sRNAs[id][4])))
            outfile.write("\n")

main()