"""
This is a script to read the csv output of
sRNARFTarget results and find the genes and
up/downstream genes in the GenBank files.
The output is in csv format so it can be
directly copied into excel
"""

import os

def getUpstream(infile):
    upstream = ""
    line = infile.readline()
    while "/product=" not in line:
        line = infile.readline()
    i = 31
    while line[i] != '"':
        if line[i] == "\n":
            line = infile.readline()
            i = 21
        upstream += line[i]
        i += 1
    return upstream
    
def getDownstream(infile):
    downstream = ""
    line = infile.readline()
    while line[5:9] != "gene":
        line = infile.readline()
    while "/product=" not in line:
        line = infile.readline()
    i = 31
    while line[i] != '"':
        if line[i] == "\n":
            line = infile.readline()
            i = 21
        downstream += line[i]
        i += 1
    return downstream

def getTarget(infile, mRNA_ID):
    infile.seek(0)
    prev = 0
    gene = ""
    while "FEATURES" not in infile.readline():
        continue
    while 1:
        line = infile.readline()
        if line[5:9] == "gene":
            curr = infile.tell()
            line = infile.readline()
            i = 21
            ids = []
            while line[i] == "/":
                ids.append(line)
                line = infile.readline()
            found = False
            for id in ids:
                if mRNA_ID in id:
                    found = True
            if found:
                line = infile.readline()
                while "/product=" not in line:
                    line = infile.readline()
                i = 31
                while line[i] != '"':
                    if line[i] == "\n":
                        line = infile.readline()
                        i = 21
                    gene += line[i]
                    i += 1
                infile.seek(prev)
                upstream = getUpstream(infile)
                infile.seek(curr)
                downstream = getDownstream(infile)
                return gene, upstream, downstream
            prev = curr
        if not line:
            break
    
def main():
    path = os.getcwd()[:-7]
    srnarf = path + "sRNARFTarget stuff/"
    directory = srnarf + "Transcriptome100-300/"
    for dir in os.listdir(directory):
        if os.path.isdir(directory + dir):
            strain = dir.split(" ")[0]
            tsv = directory + dir + "/Top_hit.tsv"
            genbank = path + "Annotated genomes/" + strain + ".gb"
            output = directory + dir + "/sRNARFTargetHits.tsv"
            infile1 = open(tsv)
            infile2 = open(genbank)
            outfile = open(output, "w")
    
            infile1.readline()
            outfile.write("sRNA_ID\tmRNA_ID\tProbability\tGene\tUpstream-gene\tDownstream-gene\n")
            num_sRNAs = 60
            for _ in range(num_sRNAs):
                line = infile1.readline().strip()
                sRNA_ID, mRNA_ID, probability = line.split(" ")
                outfile.write(sRNA_ID + "\t" + mRNA_ID + "\t" + probability + "\t")
                gene, upstream, downstream = getTarget(infile2, mRNA_ID)
                outfile.write(gene + "\t" + upstream + "\t" + downstream + "\n")
    
    infile1.close()
    infile2.close()
    outfile.close()

main()