"""
Read the data output from sRNARFTarget
and write the top 5 hits for each sRNA
into a new file and the top 1 hit into a new file.
The output files will have the top 5 and 1 hits for
each sRNA.
"""

import os
import pandas as pd
import time

def main():
    t_start = time.time()
    path = os.getcwd()[:-7]
    # destination = path + "sRNARFTarget stuff/Transcriptome_only/"
    destination = path + "sRNARFTarget stuff/Transcriptome100-300/"
    for dir in os.listdir(destination):
        if os.path.isdir(destination + dir):
            strain = dir.split(" ")[0]
            file = destination + dir + "/" + strain + "_results100-300.tsv"
            df = pd.read_csv(file, sep="\t")
    
            sRNAs = []
            # print(df.head(5))
            for index, row in df.iterrows():
                if row["sRNA_ID"] not in sRNAs:
                    sRNAs.append(row["sRNA_ID"])
            
            # Bubble sort the names to be in ascending order
            n = len(sRNAs)
            for i in range(n):
                swapped = False
                for j in range(0, n - i - 1):
                    if int(sRNAs[j][1:-1]) > int(sRNAs[j+1][1:-1]):
                        sRNAs[j], sRNAs[j+1] = sRNAs[j+1], sRNAs[j]
                        swapped = True
                if (swapped == False):
                    break

            with open(destination + dir + "/" + "Top_5_hits.tsv", "w") as file:
                for column_name in df.columns:
                    file.write(column_name + "\t")
                file.write("\n")
                for sRNA_ID in sRNAs:
                    file.write(df[df["sRNA_ID"] == sRNA_ID].head(5).to_string(index=False, header=False) + "\n")
                    
            with open(destination + dir + "/" + "Top_hit.tsv", "w") as file:
                for column_name in df.columns:
                    file.write(column_name + "\t")
                file.write("\n")
                for sRNA_ID in sRNAs:
                    file.write(df[df["sRNA_ID"] == sRNA_ID].head(1).to_string(index=False, header=False) + "\n")
                    
    t_end = time.time()
    print("Done!")
    print(f"Time to complete: {round(t_end - t_start, 2)} seconds")
                    
main()