"""
This is a script to make a transcriptome fasta
file using the genome and annotated genome files.
The output file will have all the mRNAs that
serve as input for any of the targer prediction
programs.
"""

import os
import helper_functions as hf

NUMS = "0123456789"

def main():
    path = os.getcwd()[:-7]
    fastas = path + "TVO fasta files/"
    genbanks = path + "Annotated genomes/"
    trans = path + "Transcriptomes100-300/"
    strain = "TVO_1902277"
    file1 = fastas + strain + ".fasta"
    file2 = genbanks + strain + ".gb"
    file3 = trans + strain + "_transcriptome100-300.fasta"
    infile1 = open(file1)
    infile2 = open(file2)
    outfile = open(file3, "w")
    
    count, stop = 0, 0
    while infile2.readline()[:8] != "FEATURES": continue
    while 1:
        if stop: break
        line = infile2.readline()
        if line[:6] == "ORIGIN" or not line: break
        if line[5:9] == "gene":
            # upstream, downstream = 0, 0
            upstream, downstream = 100, 300
            count += 1
            i = 0
            while line[i] not in NUMS:
                i += 1
            start, end = line[i:].strip().split("..")
            if "<" in start:
                start = start[1:]
            if ">" in end:
                end = end[1:]
            strand = "+"
            if "complement" in line:
                strand = "-"
                end = end[:-1]
            start, end = int(start), int(end)
            line = infile2.readline()
            if "gene" in line:
                gene = line.split('"')[1]
                new_locus = infile2.readline().split('"')[1]
                try:
                    old_locus = infile2.readline().split('"')[1]
                    outfile.write(">" + gene + " (" + old_locus + ")\n")
                except:
                    outfile.write(">" + gene + " (" + new_locus + ")\n")
            else:
                new_locus = line.split('"')[1]
                line = infile2.readline()
                try:
                    old_locus = line.split('"')[1]
                    outfile.write(">" + old_locus + "\n")
                except:
                    outfile.write(">" + new_locus + "\n")
            return_to = infile2.tell()
            line = infile2.readline()
            while line[5:9] != "gene":
                line = infile2.readline()
                if "ORIGIN" in line:
                    stop = 1
                    break
            next_start = ""
            j = 0
            while j < len(line) and line[j] not in NUMS: j += 1
            if j != len(line):
                while line[j] in NUMS:
                    next_start += line[j]
                    j += 1
                next_start = int(next_start)
                if next_start - end < downstream:
                    downstream = next_start - end
                infile2.seek(return_to)
            sequence = hf.retrieve_sequence(start, end, infile1, upstream, downstream)
            if strand == "-":
                sequence = hf.reverse_complement(sequence)
            outfile.write(sequence + "\n\n")
    
    infile1.close()
    infile2.close()
    outfile.close()
    
main()